// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/driver/device.c $
// $Rev: 52875 $
// $Date: 2023-04-18 08:57:44 -0500 (Tue, 18 Apr 2023) $

// 18AO8: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model			Vendor	Device	SubVen	SubDev	type
	{ "18AO8",	0x10B5, 0x9056, 0x10B5, 0x3357,	GSC_DEV_TYPE_18AO8	},
	{ NULL }
};



//*****************************************************************************
static void _channels_compute(dev_data_t* dev)
{
	long	opt	= GSC_FIELD_DECODE(dev->cache.gsc_acr_32, 16, 16);

	dev->cache.channels_max	= 8;

	switch (opt)
	{
		default:
		case 0:		dev->cache.channel_mask	= 0xFF;
					dev->cache.channel_qty	= 8;
					break;

		case 1:		dev->cache.channel_mask	= 0x0F;
					dev->cache.channel_qty	= 4;
					break;
	}
}



//*****************************************************************************
static void _fref_default_compute(dev_data_t* dev)
{
	long	opt	= GSC_FIELD_DECODE(dev->cache.gsc_acr_32, 18, 17);

	switch (opt)
	{
		default:
		case 1:
		case 2:
		case 3:
		case 0:	dev->cache.fref_default	= 40320000L;	break;
	}
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AO8_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_acr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_ACR));
		dev->vaddr.gsc_aobr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_AOBR));
		dev->vaddr.gsc_asiocr_32	= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_ASIOCR));
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_BCR));
		dev->vaddr.gsc_boor_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_BOOR));
		dev->vaddr.gsc_diopr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_DIOPR));
		dev->vaddr.gsc_obsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_OBSR));
		dev->vaddr.gsc_obtr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_OBTR));
		dev->vaddr.gsc_ocr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_OCR));
		dev->vaddr.gsc_psr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_PSR));
		dev->vaddr.gsc_rcgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_RCGR));
		dev->vaddr.gsc_rdgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_RDGR));
		dev->vaddr.gsc_scr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_SCR));
		dev->vaddr.gsc_sdbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AO8_GSC_SDBR));

		// Data cache initialization.
		dev->cache.gsc_acr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_acr_32);

		#define	NDIV_MAX			0xFFFFFFL
		#define	NDIV_MIN			80

		dev->cache.auto_cal_ms		= 8000;
		dev->cache.custom_features	= GSC_FIELD_DECODE(dev->cache.gsc_acr_32, 20, 19);
		dev->cache.fifo_size		= _256K;
		dev->cache.initialize_ms	= 3;
		dev->cache.ndiv_max			= NDIV_MAX;
		dev->cache.ndiv_min			= NDIV_MIN;
		dev->cache.selftest_ms		= 3;

		_channels_compute(dev);
		_fref_default_compute(dev);

		dev->cache.fgen_min			= dev->cache.fref_default / NDIV_MAX;
		dev->cache.fgen_max			= dev->cache.fref_default / NDIV_MIN;
		dev->cache.fsamp_min		= dev->cache.fgen_min;
		dev->cache.fsamp_max		= dev->cache.fgen_max;

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_WRITE;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


