// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/driver/irq.c $
// $Rev: 50641 $
// $Date: 2022-03-01 11:04:04 -0600 (Tue, 01 Mar 2022) $

// 18AO8: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	static const struct
	{
		s32	irq;
		s32	wait;
	} list[]	=
	{
		{ AO8_IRQ_AUTO_CAL_DONE,		AO8_WAIT_GSC_AUTO_CAL_DONE			},
		{ AO8_IRQ_SELFTEST_DATA_READY,	AO8_WAIT_GSC_SELFTEST_DATA_READY	},
		{ AO8_IRQ_DIO0_L2H,				AO8_WAIT_GSC_DIO0_L2H				},
		{ AO8_IRQ_BUF_THR_H2L,			AO8_WAIT_GSC_BUF_THR_H2L			},
		{ AO8_IRQ_BUF_THR_L2H,			AO8_WAIT_GSC_BUF_THR_L2H			},
		{ AO8_IRQ_LOAD_READY_H2L,		AO8_WAIT_GSC_LD_RDY_H2L				},
		{ AO8_IRQ_LOAD_READY_L2H,		AO8_WAIT_GSC_LD_RDY_L2H				},
		{ AO8_IRQ_BURST_READY,			AO8_WAIT_GSC_BURST_READY			},
		{ AO8_IRQ_BUF_ERROR,			AO8_WAIT_GSC_BUF_ERROR				},
		{ -1, -1 }
	};

	int	i;
	u32	irq;
	u32	psr;

	psr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_psr_32);
	irq	= psr & (psr >> 16) & 0xFFFF;

	if (irq == 0)
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
	else
	{
		// Clear the interrupt.
		psr	= psr | 0xFFFF0000;

		for (i = 0; list[i].irq >= 0; i++)
		{
			if (irq & list[i].irq)
			{
				irq	^= list[i].irq;
				psr	&= ~(list[i].irq << 16);	// Service the interrupt.
				gsc_wait_resume_irq_gsc(dev, list[i].wait);
			}
		}

		if (irq)
		{
			// Firmware has interrupts the driver doesn't support.
			gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
			psr	&= ~irq;	// Disable the interrupt(s)
		}

		// Apply the changes.
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_psr_32, psr);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_psr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_psr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_psr_32, 0);

	gsc_irq_destroy(dev);
}


