// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/utils/util_config_ao.c $
// $Rev: 52859 $
// $Date: 2023-04-17 14:38:13 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8_config_ao
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao8_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans	= 1;
	int	errs	= 0;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao8_query(fd, -1, 0, AO8_QUERY_CHANNEL_QTY, &chans);
	chans	= (chans == 4) ? 0xF : 0xFF;

	errs	+= ao8_initialize			(fd, index, verbose);

	errs	+= ao8_tx_io_mode			(fd, index, verbose, AO8_IO_MODE_DEFAULT,			NULL);
	errs	+= ao8_tx_io_overflow_data	(fd, index, verbose, AO8_TX_IO_OVERFLOW_DEFAULT,	NULL);
	errs	+= ao8_tx_io_overflow_frame	(fd, index, verbose, AO8_TX_IO_OVERFLOW_DEFAULT,	NULL);
	errs	+= ao8_tx_io_timeout		(fd, index, verbose, AO8_IO_TIMEOUT_DEFAULT,		NULL);

	errs	+= ao8_ao_config			(fd, index, verbose, AO8_AO_CONFIG_SE,				NULL);
	errs	+= ao8_buf_mode				(fd, index, verbose, AO8_BUF_MODE_OPEN,				NULL);
	errs	+= ao8_buf_thresh_level		(fd, index, verbose, 250000L,						NULL);
	errs	+= ao8_channel_sel			(fd, index, verbose, chans,							NULL);
	errs	+= ao8_data_format			(fd, index, verbose, AO8_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= ao8_irq_enable			(fd, index, verbose, 0,								NULL);
	errs	+= ao8_output_mode			(fd, index, verbose, AO8_OUTPUT_MODE_SIMULTANEOUS,	NULL);
	errs	+= ao8_range				(fd, index, verbose, AO8_RANGE_10,					NULL);

	errs	+= ao8_fsamp_ao_compute		(fd, index, verbose, fsamp, &ndiv,					NULL);
	errs	+= ao8_gen_c_ndiv			(fd, index, verbose, ndiv,							NULL);
	errs	+= ao8_gen_c_enable			(fd, index, verbose, AO8_GEN_ENABLE_YES,			NULL);
	errs	+= ao8_clock_src			(fd, index, verbose, AO8_CLOCK_SRC_RCG,				NULL);
	errs	+= ao8_clock_enable			(fd, index, verbose, AO8_CLOCK_ENABLE_YES,			NULL);

	errs	+= ao8_xcvr_output			(fd, index, verbose, AO8_XCVR_OUTPUT_ENABLE,		NULL);
	errs	+= ao8_xcvr_pol				(fd, index, verbose, AO8_XCVR_POL_DEFAULT,			NULL);
	errs	+= ao8_xcvr_type			(fd, index, verbose, AO8_XCVR_TYPE_TTL,				NULL);

	errs	+= ao8_burst_enable			(fd, index, verbose, AO8_BURST_ENABLE_NO,			NULL);
	errs	+= ao8_burst_trig_src		(fd, index, verbose, AO8_BURST_TRIG_SRC_RDG,		NULL);
	errs	+= ao8_gen_d_enable			(fd, index, verbose, AO8_GEN_ENABLE_NO,				NULL);
	errs	+= ao8_gen_d_ndiv			(fd, index, verbose, 0xFFFF,						NULL);

	errs	+= ao8_fsamp_ao_report_all	(fd, index, verbose,								NULL);

	errs	+= ao8_selftest_mode		(fd, index, verbose, AO8_SELFTEST_MODE_OUT_0,		NULL);
	errs	+= ao8_selftest_buf_clear	(fd, index, verbose);

//	errs	+= ao8_auto_calibrate		(fd, index, verbose);	// cable may need disconnecting for autocal
	errs	+= ao8_auto_cal_sts			(fd, index, verbose,								NULL);

	errs	+= ao8_buf_clear			(fd, index, verbose);
	errs	+= ao8_buf_over_data		(fd, index, verbose, AO8_BUF_OVER_DATA_CHK,			NULL);
	errs	+= ao8_buf_over_frame		(fd, index, verbose, AO8_BUF_OVER_FRAME_CHK,		NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



