// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/utils/util_reg_write.c $
// $Rev: 52859 $
// $Date: 2023-04-17 14:38:13 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8_reg_write
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8_IOCTL_REG_WRITE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		reg		The register to access.
*
*		set		This is the value to apply.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8_reg_write(int fd, int index, int verbose, u32 reg, u32 set)
{
	gsc_reg_t	arg;
	int			errs;
	int			ret;
	const char*	name;

	if (verbose)
		gsc_label_index("Register Write", index);

	arg.reg		= reg;
	arg.value	= set;
	arg.mask	= 0;	// unused here
	ret			= ao8_ioctl(fd, AO8_IOCTL_REG_WRITE, &arg);
	errs		= ret ? 1 : 0;

	if (verbose)
	{
		name	= ao8_reg_get_name(reg);
		printf(	"%s  (val 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(unsigned long) arg.value,
				name ? name : "UNKNOWN");
	}

	return(errs);
}


